import json
import requests
import os.path

from lxml.html import fromstring
from PIL import Image
from io import BytesIO


BASE_URL = 'http://127.0.0.1:8090/Fablab/fablab/Members/'
INTERESTING_FIELDS = (
    'id', 'title', 'description', 'rfid', 'credits', 'address', 'fiscalCode', 'number', 'vat'
)
IDS_TO_SCRAPE = (
    'luogo', 'interessi', 'biografia'
)


def main():
    r = requests.get(BASE_URL + 'get_children')
    users = []
    available_users = [u for u in r.json() if u not in ('user_search', 'admin')]
    for u in available_users:
        r = requests.get(BASE_URL + u + '/get_item')
        data = r.json()
        user = {}
        for field in INTERESTING_FIELDS:
            user[field] = data[field]

        r = requests.get(BASE_URL + u)
        html = fromstring(r.text)
        for html_id in IDS_TO_SCRAPE:
            selection = html.cssselect('#' + html_id)
            if not selection:
                continue
            user[html_id] = selection[0].text_content()

        selection = html.cssselect('.member-portrait img')
        if selection:
            src = selection[0].get('src')
            if not src.endswith('defaultUser.png'):
                r = requests.get(src)
                try:
                    i = Image.open(BytesIO(r.content))
                    outfile = "{}.{}".format(user['id'], i.format.lower())
                    i.save(outfile)
                except:
                    pass
                else:
                    user['immagine'] = os.path.abspath(outfile)

        users.append(user)

    with open('fablabscrape.json', 'w') as f:
        json.dump(users, f)

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-10-11 12:31
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('labadmin', '0002_auto_20160927_1249'),
    ]

    operations = [
        migrations.AddField(
            model_name='card',
            name='credits',
            field=models.IntegerField(default=0),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-10-18 14:48
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('labadmin', '0005_auto_20161012_1223'),
    ]

    operations = [
        migrations.AddField(
            model_name='device',
            name='token',
            field=models.CharField(blank=True, max_length=64, null=True),
        ),
    ]



from django import forms
from django.contrib.admin.helpers import ActionForm

from .models import Sketch, UserProfile


class DeviceActionForm(ActionForm):
    sketch = forms.ModelChoiceField(queryset=Sketch.objects.all(), empty_label=None)


class UserProfileForm(forms.ModelForm):
    class Meta:
        model = UserProfile
        fields = [
            'name', 'picture', 'address', 'tax_code', 'vat_id', 'bio'
        ]



from rest_framework.permissions import BasePermission
from rest_framework.authentication import get_authorization_header

from .models import Device


def get_token_from_request(request):
    auth = get_authorization_header(request).split()

    if len(auth) != 2:
        return None

    try:
        token = auth[1].decode()
    except UnicodeError:
        return None

    return token


class DeviceTokenPermission(BasePermission):
    def has_permission(self, request, view):
        token = get_token_from_request(request)
        if not token:
            return False

        try:
            Device.objects.get(token=token)
        except Device.DoesNotExist:
            return False

        return True



from django.contrib import admin, messages

from labadmin.models import (
    Card,
    Category,
    Device,
    Group,
    LogAccess,
    LogCredits,
    LogDevice,
    LogError,
    Payment,
    Role,
    Sketch,
    TimeSlot,
    UserProfile,
    DeviceUserCode,
)

from .forms import DeviceActionForm


class CardAdmin(admin.ModelAdmin):
    list_display = ('nfc_id', 'user', 'credits')
    search_fields = ('nfc_id', 'userprofile__user__username')
    ordering = ('-nfc_id',)

    def user(self, obj):
        try:
            return obj.userprofile.user
        except AttributeError:
            return 'n.d.'

    def save_model(self, request, obj, form, change):
        obj.save()
        obj.log_credits_update(amount=obj.credits, user=request.user, from_admin=True)

admin.site.register(Card, CardAdmin)


class DeviceUserCodeInline(admin.TabularInline):
    model = DeviceUserCode


class UserProfileAdmin(admin.ModelAdmin):
    list_display = ('name', 'card', 'displaygroups','firstSignup', 'lastSignup', 'subscription')
    ordering = ('name','-needSubscription','-endSubscription') # The negative sign indicate descendent order
    search_fields = ('card__nfc_id', 'name', 'user__username',)
    inlines = [DeviceUserCodeInline]

    def subscription(self, obj):
        return obj.endSubscription if obj.needSubscription else "lifetime membership"

admin.site.register(UserProfile, UserProfileAdmin)

class RoleAdmin(admin.ModelAdmin):
    list_display = ('name', 'valid',)
    ordering = ('name',)

admin.site.register(Role, RoleAdmin)

class GroupAdmin(admin.ModelAdmin):
    list_display = ('name',)
    ordering = ('name',)


admin.site.register(Group, GroupAdmin)


class CategoryAdmin(admin.ModelAdmin):
    list_display = ('name',)
    ordering = ('name',)

admin.site.register(Category, CategoryAdmin)

class SketchAdmin(admin.ModelAdmin):
    list_display = ('name',)
    ordering = ('name',)

admin.site.register(Sketch, SketchAdmin)


class DeviceAdmin(admin.ModelAdmin):
    list_display = ('name', 'hourlyCost', 'category', 'mac', 'last_activity')
    ordering = ('name',)
    action_form = DeviceActionForm
    actions = ['render_sketch_for_device']

    def render_sketch_for_device(self, request, queryset):
        if queryset.count() > 1:
            self.message_user(request, 'Only one device sketch can be rendered at a time', level=messages.ERROR)
            return
        sketch_id = request.POST['sketch']
        sketch = Sketch.objects.get(pk=sketch_id)
        device = queryset.first()
        return sketch.render(device, request)

admin.site.register(Device, DeviceAdmin)


class PaymentAdmin(admin.ModelAdmin):
    list_display = ('date', 'value', 'user',)
    ordering = ('-date','user',)

admin.site.register(Payment, PaymentAdmin)


class LogAccessAdmin(admin.ModelAdmin):
    list_display = ('datetime', 'user', 'card', 'opened', 'device')
    ordering = ('-datetime', 'card',)

    def user(self, obj):
        return obj.card.userprofile

admin.site.register(LogAccess,LogAccessAdmin)


class LogDeviceAdmin(admin.ModelAdmin):
    list_display = ('user', 'device', 'bootDevice', 'shutdownDevice', 'startWork', 'finishWork', 'inWorking', 'priceWork',)
    ordering = ('-bootDevice','-startWork',)

admin.site.register(LogDevice, LogDeviceAdmin)

class LogErrorAdmin(admin.ModelAdmin):
    list_display = ('datetime', 'description', 'code', 'device')
    ordering = ('-datetime',)

admin.site.register(LogError, LogErrorAdmin)

class LogCreditsAdmin(admin.ModelAdmin):
    list_display = ('datetime', 'card', 'amount', 'user', 'from_admin')
    list_filter = ('user',)

admin.site.register(LogCredits, LogCreditsAdmin)


class TimeSlotAdmin(admin.ModelAdmin):
    list_display = ('name', 'weekday_start', 'weekday_end', 'hour_start', 'hour_end',)
    ordering = ('name',)

admin.site.register(TimeSlot, TimeSlotAdmin)



import datetime
import json

from django.contrib.auth.models import User
from django.test import TestCase, Client, RequestFactory, override_settings
try:
    from django.urls import reverse
except ImportError:
    from django.core.urlresolvers import reverse
from django.utils import timezone, dateparse

from .models import (
    Card, Group, LogAccess, Role, TimeSlot, UserProfile, DeviceUserCode,
    LogCredits, Category, Device, LogDevice, LogError, Sketch
)


class TestLabAdmin(TestCase):
    @classmethod
    def setUpTestData(cls):
        full_timeslot = TimeSlot.objects.create(
            name="any day",
            weekday_start=1,
            weekday_end=7,
            hour_start=dateparse.parse_time("00:00:00"),
            hour_end=dateparse.parse_time("23:59:59"),
        )

        full_devices_role = Role.objects.create(
            name="Devices Access",
        )
        full_devices_role.time_slots.add(full_timeslot)

        devices_group = Group.objects.create(name="Full Devices access")
        devices_group.roles.add(full_devices_role)

        card = Card.objects.create(nfc_id=123456, credits=11)
        user = User.objects.create_user(username="alessandro.monaco", password="password")
        user.is_staff = True
        user.save()
        u = UserProfile.objects.create(
            user=user,
            card=card,
            name="Alessandro Monaco",
            needSubscription=False,
            endSubscription=timezone.now().date()
        )
        u.groups.add(devices_group)

        noperm_card = Card.objects.create(nfc_id=654321)
        noperm_user = User.objects.create_user(username="nopermission", password="password")
        noperm_up = UserProfile.objects.create(
            user=noperm_user,
            card=noperm_card,
            name="No Permission",
            needSubscription=True,
            endSubscription=timezone.now().date()+datetime.timedelta(days=1)
        )

        nosub_card = Card.objects.create(nfc_id=610)
        nosub_user = User.objects.create_user(username="nosubscription", password="password")
        nosub_up = UserProfile.objects.create(
            user=nosub_user,
            card=nosub_card,
            name="No subscription",
            needSubscription=True,
            endSubscription=timezone.now().date()+datetime.timedelta(days=-1)
        )
        nosub_up.groups.add(devices_group)

        category = Category.objects.create(
            name="category"
        )
        full_devices_role.categories.add(category)

        device = Device.objects.create(
            name="device",
            hourlyCost=1.0,
            category=category,
            mac="00:00:00:00:00:00"
        )

        device_user_code = DeviceUserCode.objects.create(
            code="code",
            userprofile=u,
            device=device
        )

        cls.card = card
        cls.noperm_card = noperm_card
        cls.nosub_card = nosub_card
        cls.userprofile = u
        cls.noperm_userprofile = noperm_up
        cls.nosub_userprofile = nosub_up
        cls.device = device
        cls.device_user_code = device_user_code

    def test_login_by_nfc(self):
        client = Client()
        url = reverse('nfc-users')
        data = {
            'nfc_id': self.card.nfc_id
        }
        response = client.post(url, data, format='json')
        self.assertEqual(response.status_code, 200)
        response_data = json.loads(str(response.content, encoding='utf8'))
        self.assertEqual(len(response_data), 1)
        user_profile = response_data[0]
        self.assertEqual(user_profile['id'], self.userprofile.pk)
        self.assertEqual(user_profile['name'], self.userprofile.name)

        data = {
            'nfc_id': 0
        }
        response = client.post(url, data, format='json')
        self.assertEqual(response.status_code, 400)

        response = client.get(url)
        self.assertEqual(response.status_code, 405)

    def test_open_door_by_nfc(self):
        self.assertFalse(LogAccess.objects.all().exists())

        client = Client()
        auth = 'Token {}'.format(self.device.token)
        url = reverse('open-door')
        data = {
            'nfc_id': self.card.nfc_id
        }
        response = client.post(url, data, format='json', HTTP_AUTHORIZATION=auth)
        self.assertEqual(response.status_code, 201)
        response_data = json.loads(str(response.content, encoding='utf8'))
        self.assertIn('users', response_data)
        self.assertEqual(len(response_data['users']), 1)
        user_profile = response_data['users'][0]
        self.assertEqual(user_profile['id'], self.userprofile.pk)
        self.assertEqual(user_profile['name'], self.userprofile.name)
        self.assertEqual(response_data['type'], 'other')
        self.assertIn('datetime', response_data)
        self.assertEqual(response_data['open'], self.userprofile.can_use_device_now(self.device))

        logaccess = LogAccess.objects.filter(card=self.card, device=self.device)
        self.assertTrue(logaccess.exists())

    def test_open_door_without_token(self):
        url = reverse('open-door')
        data = {
            'nfc_id': self.card.nfc_id
        }
        response = self.client.post(url, data, format='json')
        self.assertEqual(response.status_code, 403)

    def test_open_door_with_invalid_token(self):
        url = reverse('open-door')
        invalid_auth = 'Token ------'
        data = {
            'nfc_id': self.card.nfc_id
        }
        response = self.client.post(url, data, format='json', HTTP_AUTHORIZATION=invalid_auth)
        self.assertEqual(response.status_code, 403)

    def test_open_door_with_invalid_nfc(self):
        url = reverse('open-door')
        data = {
            'nfc_id': 0
        }
        auth = 'Token {}'.format(self.device.token)
        response = self.client.post(url, data, format='json', HTTP_AUTHORIZATION=auth)
        self.assertEqual(response.status_code, 400)

    def test_open_door_return_forbidden_without_subscription(self):
        self.assertFalse(LogAccess.objects.all().exists())

        client = Client()
        auth = 'Token {}'.format(self.device.token)
        url = reverse('open-door')
        data = {
            'nfc_id': self.nosub_card.nfc_id
        }
        response = client.post(url, data, format='json', HTTP_AUTHORIZATION=auth)
        self.assertEqual(response.status_code, 403)

        logaccess = LogAccess.objects.filter(card=self.nosub_card, device=self.device, opened=False)
        self.assertTrue(logaccess.exists())

    def test_open_door_does_not_handle_get_requests(self):
        url = reverse('open-door')
        auth = 'Token {}'.format(self.device.token)
        response = self.client.get(url, HTTP_AUTHORIZATION=auth)
        self.assertEqual(response.status_code, 405)

    def test_open_door_by_device_user_code(self):
        self.assertFalse(LogAccess.objects.all().exists())

        client = Client()
        auth = 'Token {}'.format(self.device.token)
        url = reverse('open-door')
        data = {
            'code': self.device_user_code.code
        }
        response = client.post(url, data, format='json', HTTP_AUTHORIZATION=auth)
        self.assertEqual(response.status_code, 201)
        response_data = json.loads(str(response.content, encoding='utf8'))
        self.assertIn('users', response_data)
        self.assertEqual(len(response_data['users']), 1)
        user_profile = response_data['users'][0]
        self.assertEqual(user_profile['id'], self.userprofile.pk)
        self.assertEqual(user_profile['name'], self.userprofile.name)
        self.assertEqual(response_data['type'], 'other')
        self.assertIn('datetime', response_data)
        self.assertEqual(response_data['open'], self.userprofile.can_use_device_now(self.device))

        logaccess = LogAccess.objects.filter(card=self.card, device=self.device)
        self.assertTrue(logaccess.exists())

    def test_open_door_by_device_user_code_invalid_code(self):
        client = Client()
        auth = 'Token {}'.format(self.device.token)
        url = reverse('open-door')
        data = {
            'code': '0'
        }
        response = client.post(url, data, format='json', HTTP_AUTHORIZATION=auth)
        self.assertEqual(response.status_code, 400)

    @override_settings(LABADMIN_NOTIFY_MQTT_ENTRANCE=True)
    def test_open_door_by_nfc_mqtt_error_log(self):
        client = Client()
        auth = 'Token {}'.format(self.device.token)
        url = reverse('open-door')
        data = {
            'nfc_id': self.card.nfc_id
        }

        mqtt_errors = LogError.objects.filter(description__endswith='failed to publish to mqtt')
        self.assertFalse(mqtt_errors.exists())
        response = client.post(url, data, format='json', HTTP_AUTHORIZATION=auth)
        mqtt_errors = LogError.objects.filter(description__endswith='failed to publish to mqtt')
        self.assertEqual(mqtt_errors.count(), 1)

    def test_get_card_credits(self):

        client = Client()
        url = reverse('card-credits')
        card = Card.objects.create(
            nfc_id=1,
            credits=1
        )
        data = {
            'nfc_id': card.nfc_id
        }

        auth = 'Token {}'.format(self.device.token)
        response = client.get(url, data, HTTP_AUTHORIZATION=auth)
        self.assertEqual(response.status_code, 200)

        response_data = json.loads(str(response.content, encoding='utf8'))
        self.assertEqual(response_data, {
            'nfc_id': card.nfc_id,
            'credits': 1
        })

        data = {
            'nfc_id': 0
        }
        response = client.get(url, data, HTTP_AUTHORIZATION=auth)
        self.assertEqual(response.status_code, 400)

        # no auth token
        response = client.get(url, data)
        self.assertEqual(response.status_code, 403)

    def test_update_card_credits(self):
        client = Client()
        auth = 'Token {}'.format(self.device.token)
        url = reverse('card-credits')
        card = Card.objects.create(
            nfc_id=1,
            credits=10
        )

        self.assertEqual(LogCredits.objects.count(), 0)

        # not enough credits
        data = {
            'nfc_id': card.nfc_id,
            'amount': -20
        }
        response = client.post(url, data, HTTP_AUTHORIZATION=auth)
        self.assertEqual(response.status_code, 403)

        # can't add credits
        data = {
            'nfc_id': card.nfc_id,
            'amount': 10
        }
        response = client.post(url, data, HTTP_AUTHORIZATION=auth)
        self.assertEqual(response.status_code, 400)

        # consume credits
        data = {
            'nfc_id': card.nfc_id,
            'amount': -10
        }
        response = client.post(url, data, HTTP_AUTHORIZATION=auth)
        self.assertEqual(response.status_code, 200)

        response_data = json.loads(str(response.content, encoding='utf8'))
        self.assertEqual(response_data, {
            'nfc_id': 1,
            'credits': 0
        })

        self.assertEqual(LogCredits.objects.count(), 1)

        data = {
            'nfc_id': 0
        }
        response = client.post(url, data, HTTP_AUTHORIZATION=auth)
        self.assertEqual(response.status_code, 400)

        data = {
            'nfc_id': 1,
            'amount': 'amount'
        }
        response = client.post(url, data, HTTP_AUTHORIZATION=auth)
        self.assertEqual(response.status_code, 400)

        # invalid token
        invalid_auth = 'Token ------'
        response = client.post(url, data, format='json', HTTP_AUTHORIZATION=invalid_auth)
        self.assertEqual(response.status_code, 403)

        # no auth
        data = {
            'nfc_id': 0
        }
        response = client.post(url, data)
        self.assertEqual(response.status_code, 403)

    def test_logcredits_str(self):
        logcredits = LogCredits.objects.create(
            card=self.card,
            amount=10,
            user=self.userprofile.user,
            from_admin=False
        )
        self.assertIn('updated credits for card', str(logcredits))

        logcredits.from_admin = True
        logcredits.save()
        self.assertIn('set credits for card', str(logcredits))

    def test_device_token_creation(self):
        category = Category.objects.create(
            name="category"
        )

        device = Device.objects.create(
            name="device",
            hourlyCost=1.0,
            category=category,
            mac="00:00:00:00:00:00"
        )
        self.assertTrue(device.token)

        # token is created once
        old_token = device.token
        device.save()
        device = Device.objects.get(pk=device.pk)
        self.assertEqual(old_token, device.token)

    def test_device_stop(self):
        now = timezone.now()
        logdevice = LogDevice.objects.create(
            device=self.device,
            user=self.userprofile,
            bootDevice=now,
            startWork=now,
            shutdownDevice=now,
            finishWork=now,
            hourlyCost=self.device.hourlyCost,
        )
        logdevice.stop()
        self.assertEqual(logdevice.startWork, logdevice.bootDevice)
        self.assertTrue(logdevice.startWork < logdevice.finishWork)
        self.assertEqual(logdevice.finishWork, logdevice.shutdownDevice)

    def test_logdevice_pricework(self):
        now = timezone.now()
        logdevice = LogDevice.objects.create(
            device=self.device,
            user=self.userprofile,
            bootDevice=now,
            startWork=now,
            shutdownDevice=now,
            finishWork=now,
            hourlyCost=self.device.hourlyCost,
        )

        self.assertEqual(logdevice.priceWork(), 'inWorking...')
        logdevice.stop()
        logdevice.finishWork = logdevice.finishWork + datetime.timedelta(seconds=60)
        logdevice.save()
        self.assertEqual(logdevice.priceWork(), 0.02)

    def test_device_last_activity(self):
        self.assertEqual(self.device.last_activity(), '')

        now = timezone.now()
        logdevice = LogDevice.objects.create(
            device=self.device,
            user=self.userprofile,
            bootDevice=now,
            startWork=now,
            shutdownDevice=now,
            finishWork=now,
            hourlyCost=self.device.hourlyCost,
        )

        self.assertTrue(logdevice.inWorking)
        self.assertTrue(self.device.last_activity() >= logdevice.startWork)

        logdevice.stop()
        self.assertFalse(logdevice.inWorking)
        self.assertEqual(self.device.last_activity(), logdevice.finishWork)

    def test_device_last_activity_with_logaccess(self):
        logaccess = LogAccess.objects.log(
            card=self.card,
            opened=True,
            device=self.device
        )
        now = timezone.now()
        logdevice = LogDevice.objects.create(
            device=self.device,
            user=self.userprofile,
            bootDevice=now,
            startWork=now,
            shutdownDevice=now,
            finishWork=now,
            hourlyCost=self.device.hourlyCost,
        )
        self.assertIn('enter permitted', self.device.last_activity())

    def test_render_device_sketch(self):
        sketch = Sketch.objects.create(
            name='nome sketch',
            code='{{ device.name }} {{ user.name }}',
            file_format='.ino',
        )
        category = Category.objects.create(
            name="category"
        )
        device = Device.objects.create(
            name='device',
            hourlyCost=1.0,
            category=category,
            mac='00:00:00:00:00:00',
        )
        factory = RequestFactory()
        request = factory.get('/admin/labadmin/device/')
        request.user = self.card.userprofile.user
        response = sketch.render(device, request)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response['Content-Type'], 'text/plain')
        self.assertIn('filename="device-nome-sketch.ino"', response['Content-Disposition'])
        self.assertContains(response, 'device Alessandro Monaco')

    def test_device_start_use(self):
        client = Client()
        auth = 'Token {}'.format(self.device.token)
        url = reverse('device-use-start')

        self.assertFalse(LogDevice.objects.filter(device=self.device).exists())

        # not authenticated
        data = {
            'nfc_id': self.card.nfc_id,
        }
        response = client.post(url, data)
        self.assertEqual(response.status_code, 403)

        # invalid data
        data = {
        }
        response = client.post(url, data, HTTP_AUTHORIZATION=auth)
        self.assertEqual(response.status_code, 400)

        data = {
            'nfc_id': 0
        }
        response = client.post(url, data, HTTP_AUTHORIZATION=auth)
        self.assertEqual(response.status_code, 400)

        # not enough permissions
        data = {
            'nfc_id': self.noperm_card.nfc_id,
        }
        response = client.post(url, data, HTTP_AUTHORIZATION=auth)
        self.assertEqual(response.status_code, 400)

        # ok
        data = {
            'nfc_id': self.card.nfc_id,
        }
        response = client.post(url, data, HTTP_AUTHORIZATION=auth)
        self.assertEqual(response.status_code, 201)
        self.assertEqual(LogDevice.objects.count(), 1)
        self.assertTrue(
            LogDevice.objects.filter(device=self.device, user=self.userprofile, inWorking=True).exists()
        )
        self.assertJSONEqual(response.content.decode('utf-8'), {
            'cost': self.device.hourlyCost
        })

        # ok too but the previous LogDevice is closed
        data = {
            'nfc_id': self.card.nfc_id,
        }
        response = client.post(url, data, HTTP_AUTHORIZATION=auth)
        self.assertEqual(response.status_code, 201)
        self.assertEqual(LogDevice.objects.count(), 2)
        self.assertEqual(
            LogDevice.objects.filter(device=self.device, user=self.userprofile, inWorking=True).count(), 1
        )

        # invalid token
        invalid_auth = 'Token ------'
        response = client.post(url, data, format='json', HTTP_AUTHORIZATION=invalid_auth)
        self.assertEqual(response.status_code, 403)

    def test_device_start_works_with_code(self):
        auth = 'Token {}'.format(self.device.token)
        url = reverse('device-use-start')
        data = {
            'code': self.device_user_code.code,
        }
        response = self.client.post(url, data, HTTP_AUTHORIZATION=auth)
        self.assertEqual(response.status_code, 201)
        self.assertEqual(LogDevice.objects.count(), 1)
        self.assertTrue(
            LogDevice.objects.filter(
                device=self.device, user=self.userprofile, inWorking=True
            ).exists()
        )
        self.assertJSONEqual(response.content.decode('utf-8'), {
            'cost': self.device.hourlyCost
        })

    def test_device_start_fails_with_invalid_code(self):
        auth = 'Token {}'.format(self.device.token)
        url = reverse('device-use-start')
        data = {
        }
        response = self.client.post(url, data, HTTP_AUTHORIZATION=auth)
        self.assertEqual(response.status_code, 400)

        data = {
            'code': ''
        }
        response = self.client.post(url, data, HTTP_AUTHORIZATION=auth)
        self.assertEqual(response.status_code, 400)

    def test_device_stop_use(self):
        client = Client()
        auth = 'Token {}'.format(self.device.token)
        url = reverse('device-use-stop')

        self.assertFalse(LogDevice.objects.filter(device=self.device).exists())

        # not authenticated
        data = {
            'nfc_id': self.card.nfc_id,
        }
        response = client.post(url, data)
        self.assertEqual(response.status_code, 403)

        # invalid data
        data = {
        }
        response = client.post(url, data, HTTP_AUTHORIZATION=auth)
        self.assertEqual(response.status_code, 400)

        data = {
            'nfc_id': 0
        }
        response = client.post(url, data, HTTP_AUTHORIZATION=auth)
        self.assertEqual(response.status_code, 400)

        # no open logdevice
        data = {
            'nfc_id': self.card.nfc_id,
        }
        response = client.post(url, data, HTTP_AUTHORIZATION=auth)
        self.assertEqual(response.status_code, 400)

        now = timezone.now()
        LogDevice.objects.create(
            device=self.device,
            user=self.userprofile,
            startWork=now,
            bootDevice=now,
            shutdownDevice=now,
            finishWork=now,
            hourlyCost="0.0"
        )

        # ok
        data = {
            'nfc_id': self.card.nfc_id,
        }
        response = client.post(url, data, HTTP_AUTHORIZATION=auth)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(LogDevice.objects.count(), 1)
        self.assertTrue(
            LogDevice.objects.filter(device=self.device, user=self.userprofile, inWorking=False).exists()
        )
        self.assertJSONEqual(response.content.decode('utf-8'), {
            'cost': 0
        })

        # invalid token
        invalid_auth = 'Token ------'
        response = client.post(url, data, format='json', HTTP_AUTHORIZATION=invalid_auth)
        self.assertEqual(response.status_code, 403)

    def test_device_stop_works_with_code(self):
        now = timezone.now()
        LogDevice.objects.create(
            device=self.device,
            user=self.userprofile,
            startWork=now,
            bootDevice=now,
            shutdownDevice=now,
            finishWork=now,
            hourlyCost="0.0"
        )
        auth = 'Token {}'.format(self.device.token)
        url = reverse('device-use-stop')
        data = {
            'code': self.device_user_code.code,
        }
        response = self.client.post(url, data, HTTP_AUTHORIZATION=auth)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(LogDevice.objects.count(), 1)
        self.assertTrue(
            LogDevice.objects.filter(
                device=self.device, user=self.userprofile, inWorking=False
            ).exists()
        )
        self.assertJSONEqual(response.content.decode('utf-8'), {
            'cost': 0
        })

    def test_device_stop_fails_with_invalid_code(self):
        auth = 'Token {}'.format(self.device.token)
        url = reverse('device-use-stop')
        data = {
        }
        response = self.client.post(url, data, HTTP_AUTHORIZATION=auth)
        self.assertEqual(response.status_code, 400)

        data = {
            'code': ''
        }
        response = self.client.post(url, data, HTTP_AUTHORIZATION=auth)
        self.assertEqual(response.status_code, 400)

    def test_user_can_use_device_now(self):
        self.assertTrue(self.userprofile.can_use_device_now(self.device))

    def test_user_can_use_device_now_invalid_role(self):
        timeslot = TimeSlot.objects.create(
            name="any day for invalid role",
            weekday_start=1,
            weekday_end=7,
            hour_start=dateparse.parse_time("00:00:00"),
            hour_end=dateparse.parse_time("23:59:59"),
        )

        role = Role.objects.create(
            name="invalid role",
            valid=False
        )
        role.time_slots.add(timeslot)

        group = Group.objects.create(name="group with invalid role")
        group.roles.add(role)
        self.noperm_userprofile.groups.add(group)
        self.assertFalse(self.noperm_userprofile.can_use_device_now(self.device))

    def test_user_can_use_device_now_no_group(self):
        self.assertFalse(self.noperm_userprofile.can_use_device_now(self.device))

    def test_user_can_use_device_role_not_categories(self):
        timeslot = TimeSlot.objects.create(
            name="any day",
            weekday_start=1,
            weekday_end=7,
            hour_start=dateparse.parse_time("00:00:00"),
            hour_end=dateparse.parse_time("23:59:59"),
        )

        role = Role.objects.create(
            name="role",
        )
        role.time_slots.add(timeslot)

        group = Group.objects.create(name="group")
        group.roles.add(role)
        self.noperm_userprofile.groups.add(group)
        self.assertFalse(self.noperm_userprofile.can_use_device_now(self.device))

    def test_user_cannot_use_device_now_without_valid_subscription_but_valid_timeslot(self):
        self.assertTrue(self.nosub_userprofile.needSubscription)
        self.assertEqual(self.nosub_userprofile.displaygroups(), 'Full Devices access')
        self.assertFalse(self.nosub_userprofile.can_use_device_now(self.device))

    def test_logaccess_print(self):
        log = LogAccess.objects.log(
            card=self.card,
            opened=False,
            device=self.device
        )
        self.assertIn('enter not permitted', str(log))

        log = LogAccess.objects.log(
            card=self.card,
            opened=True,
            device=self.device
        )
        self.assertIn('enter permitted', str(log))

    def test_user_profile_view_no_login(self):
        url = reverse('labadmin-user-profile')
        response = self.client.get(url)
        self.assertRedirects(response, '/accounts/login/?next={}'.format(url))

    def test_user_profile_view_show_user_credits(self):
        url = reverse('labadmin-user-profile')
        self.client.login(username='alessandro.monaco', password='password')

        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, ': 11')

    def test_user_profile_view_update(self):
        url = reverse('labadmin-user-profile')
        self.client.login(username='alessandro.monaco', password='password')

        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, 'Alessandro Monaco')

        data = {
            'name': 'Ciccio Pasticcio',
            'address': 'some address',
            'tax_code': '456',
            'vat_id': '123',
            'bio': 'A poignant bio',
        }
        response = self.client.post(url, data)
        self.assertRedirects(response, url)

        response = self.client.get(url)
        self.assertContains(response, 'Ciccio Pasticcio')
        self.assertContains(response, 'some address')
        self.assertContains(response, '456')
        self.assertContains(response, '123')
        self.assertContains(response, 'A poignant bio')
        self.client.logout()

    def test_has_valid_subscription_works_with_needsubscription_false(self):
        self.assertFalse(self.userprofile.needSubscription)
        self.assertTrue(self.userprofile.has_valid_subscription())

    def test_has_valid_subscription_works_with_needsubscription_and_endsubscription_in_the_future(self):
        self.assertTrue(self.noperm_userprofile.needSubscription)
        self.assertTrue(self.noperm_userprofile.has_valid_subscription())

    def test_has_valid_subscription_fails_with_needsubscription_and_endsubscription_in_the_past(self):
        self.assertTrue(self.nosub_userprofile.needSubscription)
        self.assertFalse(self.nosub_userprofile.has_valid_subscription())


class TimeSlotTests(TestCase):
    def test_timeslot_manager_now(self):
        now = timezone.now()
        now_time = now.time()
        now_weekday = now.isoweekday()

        # enough for the tests to work :)
        end = now + datetime.timedelta(minutes=1)
        end_time = end.time()
        end_weekday = now.isoweekday()

        self.assertFalse(TimeSlot.objects.can_now().exists())

        open_ts = TimeSlot.objects.create(
            hour_start=now_time,
            hour_end=end_time,
            weekday_start=now_weekday,
            weekday_end=end_weekday
        )

        closed_ts_weekday = TimeSlot.objects.create(
            hour_start=now_time,
            hour_end=end_time,
            weekday_start=end_weekday+1,
            weekday_end=end_weekday+1
        )

        closed_ts_hour = TimeSlot.objects.create(
            hour_start=now_time,
            hour_end=now_time,
            weekday_start=now_weekday,
            weekday_end=end_weekday
        )

        ts_now = TimeSlot.objects.can_now()
        self.assertTrue(ts_now.exists())
        self.assertEqual(ts_now.count(), 1)
        self.assertEqual(ts_now.first().pk, open_ts.pk)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2017-03-02 10:31
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('labadmin', '0009_logerror_device'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='logaccess',
            name='users',
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2017-01-18 19:12
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('labadmin', '0006_device_token'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='role',
            name='role_kind',
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-05-20 19:59
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('labadmin', '0012_auto_20170518_2158'),
    ]

    operations = [
        migrations.CreateModel(
            name='DeviceUserCode',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', models.CharField(max_length=64)),
                ('device', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='labadmin.Device')),
                ('userprofile', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='labadmin.UserProfile')),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='deviceusercode',
            unique_together=set([('userprofile', 'device', 'code')]),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-09-27 12:49
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('labadmin', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Card',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('nfc_id', models.BigIntegerField(unique=True)),
            ],
        ),
        migrations.RemoveField(
            model_name='logaccess',
            name='user',
        ),
        migrations.RemoveField(
            model_name='userprofile',
            name='nfcId',
        ),
        migrations.AddField(
            model_name='logaccess',
            name='users',
            field=models.ManyToManyField(to='labadmin.UserProfile'),
        ),
        migrations.AddField(
            model_name='logaccess',
            name='card',
            field=models.ForeignKey(default=1, on_delete=django.db.models.deletion.CASCADE, to='labadmin.Card'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='userprofile',
            name='card',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='labadmin.Card'),
        ),
    ]



from django.conf.urls import url
from django.contrib.auth import views as auth_views
from django.views.generic import TemplateView

from . import views


pwd_change_opts = {
    'template_name': 'labadmin/password_change_form.html',
    'post_change_redirect': 'labadmin-password-change-done',
}

pwd_change_done_opts = {
    'template_name': 'labadmin/password_change_done.html',
}

pwd_reset_opts = {
    'template_name': 'labadmin/password_reset_form.html',
    'email_template_name': 'labadmin/password_reset_email.html',
    'post_reset_redirect': 'labadmin-password-reset-done',
}

pwd_reset_done_opts = {
    'template_name': 'labadmin/password_reset_done.html',
}

pwd_reset_confirm_opts = {
    'template_name': 'labadmin/password_reset_confirm.html',
    'post_reset_redirect': 'labadmin-password-reset-complete',
}

pwd_reset_complete_opts = {
    'template_name': 'labadmin/password_reset_complete.html',
}

urlpatterns = [
    url(r'^opendoorbynfc/$', views.OpenDoor.as_view()),
    url(r'^opendoor/$', views.OpenDoor.as_view(), name="open-door"),
    url(r'^user/identity/$', views.UserIdentity.as_view()),
    url(r'^nfc/users/$', views.LoginByNFC.as_view(), name='nfc-users'),
    url(r'^card/credits/$', views.CardCredits.as_view(), name='card-credits'),
    url(r'^device/use/start/$', views.DeviceStartUse.as_view(), name='device-use-start'),
    url(r'^device/use/stop/$', views.DeviceStopUse.as_view(), name='device-use-stop'),

    url(r'^$', TemplateView.as_view(template_name='labadmin/index.html'), name='labadmin-index'),
    url(r'^profile$', views.UserProfileView.as_view(), name='labadmin-user-profile'),
    url(r'^password/change$', auth_views.password_change, pwd_change_opts, name='labadmin-password-change'),
    url(r'^password/change/done$', auth_views.password_change_done, pwd_change_done_opts, name='labadmin-password-change-done'),
    url(r'^password/reset$', auth_views.password_reset, pwd_reset_opts, name='labadmin-password-reset'),
    url(r'^password/reset/done$', auth_views.password_reset_done, pwd_reset_done_opts, name='labadmin-password-reset-done'),
    url(r'^password/reset/reset/(?P<uidb64>[0-9A-Za-z_\-]+)/(?P<token>[0-9A-Za-z]{1,13}-[0-9A-Za-z]{1,20})/$', auth_views.password_reset_confirm, pwd_reset_confirm_opts, name='labadmin-password-reset-confirm'),
    url(r'^password/reset/complete$', auth_views.password_reset_complete, pwd_reset_complete_opts, name='labadmin-password-reset-complete'),
    url(r'^logout$', auth_views.logout, {'next_page': 'labadmin-index'}, name='labadmin-logout'),
 ]



import json

from django.contrib.auth.decorators import login_required
from django.contrib import messages
from django.http import Http404
from django.shortcuts import get_object_or_404, redirect
from django.utils.translation import ugettext as _
from django.db import transaction, IntegrityError
from django.conf import settings
from django.utils import timezone
from django.utils.decorators import method_decorator
from django.views.generic import View
from django.views.generic.base import TemplateResponseMixin

from labadmin.forms import UserProfileForm
from labadmin.serializers import (
    UserProfileSerializer, CardSerializer, CardUpdateSerializer
)
from labadmin.models import (
    Card,
    Device,
    Group,
    LogAccess,
    LogDevice,
    LogError,
    UserProfile,
    DeviceUserCode
)
from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework import status

from oauth2_provider.models import AccessToken

from .notifications import mqtt_publish
from .permissions import (
    get_token_from_request, DeviceTokenPermission
)


MQTT_ENTRANCE_TOPIC = getattr(settings, 'LABADMIN_MQTT_ENTRANCE_TOPIC', 'labadmin/entrance')


class LoginByNFC(APIView):
    """
    API For Login (return user informations)
    In order to use this API send a POST with a value named 'nfc_id'
    The return value is a json array with users associated to the card

    If the nfc code isn't correct or valid, the API save in 'LogError' a new error that contains the error then return an error message to client (HTTP_400_BAD_REQUEST)
    """

    def post(self, request, format=None):
        nfc = request.data.get('nfc_id')
        users = UserProfile.objects.filter(card__nfc_id=nfc)
        if not users.exists():
            LogError(description="Api: Login By NFC - NFC not Valid", code=nfc).save()
            return Response("", status=status.HTTP_400_BAD_REQUEST)
        return Response(UserProfileSerializer(users, many=True).data, status=status.HTTP_200_OK)


class GetUserFromNFCMixin(object):
    def get_card_user_from_nfc_id(self, nfc_id):
        try:
            card = Card.objects.get(nfc_id=nfc_id)
            user = card.userprofile
        except Card.DoesNotExist:
            card = None
            user = None
        return card, user


class GetUserFromDeviceUserCodeMixin(object):
    def get_card_user_from_code(self, code, device):
        try:
            device_user_code = DeviceUserCode.objects.get(code=code, device=device)
            user = device_user_code.userprofile
            card = user.card
        except DeviceUserCode.DoesNotExist:
            card = None
            user = None
        return card, user


class OpenDoor(APIView, GetUserFromNFCMixin, GetUserFromDeviceUserCodeMixin):
    """
    API For Opening the Door
    In order to use this API send a POST with one of:
        - 'nfc_id': nfc id as integer
        - 'code': 64 char max code
    The nfc_id will match a Card, the code will match a DeviceUserCode.
    The return value is a json array with:
        'users': an object with 'id' and 'name' of users
        'utype': 'fablab' if there's a user in the 'Fablab'group or 'other'
        'datetime': datetime objects of the current time
        'open': return if the user can open the door or not

    If the nfc code isn't correct or valid, the API save in 'LogError' a new error that contains the error then return an alert message to client (HTTP_400_BAD_REQUEST)
    If the card does not have permissions to open the door a status code of 403 is returned to the user
    """

    permission_classes = (DeviceTokenPermission,)

    def post(self, request, format=None):
        token = get_token_from_request(request)
        try:
            device = Device.objects.get(token=token)
        except Device.DoesNotExist:
            LogError(description="Api: Open Door - token not valid", code=token or '').save()
            return Response("", status=status.HTTP_400_BAD_REQUEST)

        nfc_id = request.data.get('nfc_id')
        card, user = self.get_card_user_from_nfc_id(nfc_id)

        if user is None:
            code = request.data.get('code')
            card, user = self.get_card_user_from_code(code, device)

        if user is None:
            if nfc_id:
                LogError.objects.create(
                    description="Api: Open Door - nfc ID not valid", code=nfc_id, device=device
                )
            else:
                LogError.objects.create(
                    description="Api: Open Door - device code not valid",
                    code=code or '',
                    device=device
                )
            return Response("", status=status.HTTP_400_BAD_REQUEST)

        can_open = user.can_use_device_now(device)
        users = UserProfile.objects.filter(pk=user.pk)
        log_access = LogAccess.objects.log(card=card, opened=can_open, device=device)
        if not can_open:
            return Response("", status=status.HTTP_403_FORBIDDEN)

        users_pks = users.values_list('pk', flat=True)
        if Group.objects.filter(userprofile__in=users_pks, name__icontains='Fablab').exists():
            utype = "fablab"
        else:
            utype = "other"

        MQTT_ENTRANCE_NOTIFICATION = getattr(settings, 'LABADMIN_NOTIFY_MQTT_ENTRANCE', False)
        if MQTT_ENTRANCE_NOTIFICATION and log_access.opened:
            payload = {
                'user': user.name,
                'state': _('entrance')
            }
            try:
                mqtt_publish(MQTT_ENTRANCE_TOPIC, json.dumps(payload))
            except Exception as e:
                LogError(description="Api: Open Door By NFC - failed to publish to mqtt", code=e, device=device).save()

        data = {
            "users": UserProfileSerializer(users, many=True).data,
            "type": utype,
            "datetime": log_access.datetime,
            "open": log_access.opened
        }
        return Response(data, status=status.HTTP_201_CREATED)


class DeviceStartUse(APIView, GetUserFromNFCMixin, GetUserFromDeviceUserCodeMixin):
    """
    API to track the start of a device usage by an user.
    In case of success returns a json HTTP 201
    Adds a new 'LogDevice' object if the user can use the device, otherwise returns an error.

    In order to use this API send a POST with one of:
        - 'nfc_id': nfc id as integer
        - 'code': code, max 64 char
    The nfc_id will match a Card, the code will match a DeviceUserCode.
    """

    permission_classes = (DeviceTokenPermission,)

    def post(self, request, format=None):
        token = get_token_from_request(request)
        try:
            device = Device.objects.get(token=token)
        except Device.DoesNotExist:
            LogError(description="Api: Use Device - token not valid", code=token or '').save()
            return Response("", status=status.HTTP_400_BAD_REQUEST)

        nfc_id = request.data.get('nfc_id')
        card, user = self.get_card_user_from_nfc_id(nfc_id)

        if user is None:
            code = request.data.get('code')
            card, user = self.get_card_user_from_code(code, device)

        if user is None:
            if nfc_id:
                LogError.objects.create(
                    description="Api: Use Device - nfc ID not valid", code=nfc_id, device=device
                )
            else:
                LogError.objects.create(
                    description="Api: Use Device - code not valid", code=code or '', device=device
                )
            return Response("", status=status.HTTP_400_BAD_REQUEST)

        if not user.can_use_device_now(device):
            LogError.objects.create(
                description="Api: Use Device - card {} can't use device {}".format(nfc_id, token),
                device=device
            )
            return Response("", status=status.HTTP_400_BAD_REQUEST)

        # cleanup leaked instances
        log = LogDevice.objects.filter(device=device, inWorking=True)
        for ll in log:
            ll.stop()

        now = timezone.now()
        log = LogDevice.objects.create(
            device=device, user=user, startWork=now, bootDevice=now,
            shutdownDevice=now + timezone.timedelta(seconds=10),
            finishWork=now + timezone.timedelta(seconds=10),
            hourlyCost=device.hourlyCost
        )
        return Response(
            {"cost": log.hourlyCost}, status=status.HTTP_201_CREATED
        )


class DeviceStopUse(APIView, GetUserFromNFCMixin, GetUserFromDeviceUserCodeMixin):
    """
    API to track the stop of a device usage by an user.
    In case of success returns a json HTTP 200
    Updates the 'LogDevice' object for the user on this device, otherwise returns an error.

    In order to use this API send a POST with one of:
        - 'nfc_id': nfc id as integer
        - 'code': code, max 64 char
    The nfc_id will match a Card, the code will match a DeviceUserCode.
    """

    permission_classes = (DeviceTokenPermission,)

    def post(self, request, format=None):
        token = get_token_from_request(request)
        try:
            device = Device.objects.get(token=token)
        except Device.DoesNotExist:
            LogError(description="Api: Stop use Device - token not valid", code=token or '').save()
            return Response("", status=status.HTTP_400_BAD_REQUEST)


        nfc_id = request.data.get('nfc_id')
        card, user = self.get_card_user_from_nfc_id(nfc_id)

        if user is None:
            code = request.data.get('code')
            card, user = self.get_card_user_from_code(code, device)

        if user is None:
            if nfc_id:
                LogError.objects.create(
                    description="Api: Stop use Device - nfc ID not valid", code=nfc_id, device=device
                )
            else:
                LogError.objects.create(
                    description="Api: Stop use Device - code not valid", code=code or '', device=device
                )
            return Response("", status=status.HTTP_400_BAD_REQUEST)

        try:
            log = LogDevice.objects.get(device=device, user=user, inWorking=True)
        except LogDevice.DoesNotExist:
            code = "card {} for device with token {}".format(nfc_id, token)
            LogError(description="Api: Stop use Device - no device log found", code=code, device=device).save()
            return Response("", status=status.HTTP_400_BAD_REQUEST)

        log.stop()

        return Response({"cost": log.priceWork()}, status=status.HTTP_200_OK)


class UserIdentity(APIView):
    def get(self, request, format=None):
        token_string = request.GET.get('access_token')
        now = timezone.now()
        token = get_object_or_404(AccessToken, token=token_string, expires__gt=now, user__isnull=False)
        user = token.user
        if not user.is_active:
            raise Http404

        try:
            up = user.userprofile
        except UserProfile.DoesNotExist:
            up = None

        content = {
            'id': user.pk,
            'username': user.username,
            'email': user.email,
            'is_staff': user.is_staff,
        }

        if up:
            meta = request.META
            if meta['SERVER_PORT'] and meta['SERVER_PORT'] not in ('443', '80'):
                port = ':{}'.format(meta['SERVER_PORT'])
            else:
                port = ''
            base_url = '{}://{}{}'.format(request.scheme, meta['SERVER_NAME'], port)
            groups = up.groups.values_list('name', flat=True)
            content.update({
                'name': up.name,
                'avatar_url': '{}{}'.format(base_url, up.picture.url) if up.picture else '',
                'bio': up.bio or '',
                'groups': groups,
            })
        return Response(content)

class CardCredits(APIView):
    """
    API for getting card credits
    In order to use this API send a GET with:
    - 'nfc_id', to identifiy the card

    The return value is a json array with the nfc_id and the credits amount for the card.

    If the nfc code isn't valid a 'LogError' instance is saved and returns 400 status code
    """

    permission_classes = (DeviceTokenPermission,)

    def get(self, request, format=None):
        token = get_token_from_request(request)
        try:
            device = Device.objects.get(token=token)
        except Device.DoesNotExist:
            LogError(description="Api: Update Card Credits - token not valid", code=token or '').save()
            return Response("", status=status.HTTP_400_BAD_REQUEST)

        nfc = request.query_params.get('nfc_id')
        try:
            card = Card.objects.get(nfc_id=nfc)
        except Card.DoesNotExist:
            LogError(description="Api: Update Card Credits - NFC not Valid", code=nfc or '', device=device).save()
            return Response("", status=status.HTTP_400_BAD_REQUEST)
        return Response(CardSerializer(card).data, status=status.HTTP_200_OK)

    """
    API for updating card credits
    In order to use this API send a POST with:
    - 'nfc_id', to identifiy the card
    - 'amount', integer to add to credit amount

    The return value is a json array with the nfc_id and new credits amount for the card.

    The amount must be negative and if there's not enough credits it'll returns 403 status code
    If the amount or the nfc code isn't valid a 'LogError' instance is saved and status code 400 returned
    """
    def post(self, request, format=None):
        token = get_token_from_request(request)
        try:
            device = Device.objects.get(token=token)
        except Device.DoesNotExist:
            LogError(description="Api: Update Card Credits - token not valid", code=token or '').save()
            return Response("", status=status.HTTP_400_BAD_REQUEST)

        serializer = CardUpdateSerializer(data=request.data)
        nfc = serializer.initial_data.get('nfc_id')
        if not serializer.is_valid():
            LogError(description="Api: Update Card Credits - Invalid data", code=nfc or '', device=device).save()
            return Response("", status=status.HTTP_400_BAD_REQUEST)

        try:
            # avoid race conditions updating the card model
            with transaction.atomic():
                try:
                    card = Card.objects.get(nfc_id=nfc)
                except Card.DoesNotExist:
                    LogError(description="Api: Update Card Credits - NFC not Valid", code=nfc, device=device).save()
                    return Response("", status=status.HTTP_400_BAD_REQUEST)

                amount = serializer.data['amount']
                if amount >= 0:
                    msg = "Api: Update Card Credits - amount can only be negative: {}".format(
                        amount
                    )
                    LogError(description=msg, code=nfc, device=device).save()
                    return Response("", status=status.HTTP_400_BAD_REQUEST)

                new_amount = card.credits + amount
                if new_amount < 0:
                    msg = "Api: Update Card Credits - Not enough credits: requested {} of {} available".format(
                        amount, card.credits
                    )
                    LogError(description=msg, code=nfc, device=device).save()
                    return Response("", status=status.HTTP_403_FORBIDDEN)

                # update the card with the new credits amount
                card.credits = new_amount
                card.save(update_fields=['credits'])
                user = request.user if request.user.is_authenticated() else None
                card.log_credits_update(amount=amount, user=user)

        except IntegrityError:
            LogError(description="Api: Update Card Credits - IntegrityError", code=nfc, device=device).save()
            return Response("", status=status.HTTP_400_BAD_REQUEST)

        return Response(CardSerializer(card).data, status=status.HTTP_200_OK)


class UserProfileView(TemplateResponseMixin, View):
    template_name = 'labadmin/userprofile.html'

    def get(self, request, *args, **kwargs):
        try:
            up = request.user.userprofile
        except UserProfile.DoesNotExist:
            up = None
        form = UserProfileForm(instance=up)
        return self.render_to_response({'form': form, 'user': request.user})

    def post(self, request, *args, **kwargs):
        try:
            up = request.user.userprofile
        except UserProfile.DoesNotExist:
            up = None
        form = UserProfileForm(request.POST, request.FILES, instance=up)
        if form.is_valid():
            up = form.save(commit=False)
            up.user = request.user
            if not up.endSubscription:
                up.endSubscription = timezone.now()
            up.save()
            messages.add_message(request, messages.INFO, 'Profile updated correctly')
            return redirect('labadmin-user-profile')
        return self.render_to_response({'form': form, 'user': request.user})

    @method_decorator(login_required)
    def dispatch(self, *args, **kwargs):
        return super(UserProfileView, self).dispatch(*args, **kwargs)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2017-03-02 12:09
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('labadmin', '0010_remove_logaccess_users'),
    ]

    operations = [
        migrations.CreateModel(
            name='Sketch',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100)),
                ('code', models.TextField()),
                ('file_format', models.CharField(default='.txt', max_length=10)),
            ],
            options={
                'verbose_name_plural': 'Sketches',
            },
        ),
    ]



# -*- coding: utf-8 -*-
import json
import os.path

from django.contrib.auth.models import User
from django.core.files import File
from django.utils import timezone

from labadmin.models import UserProfile, Card


def import_fablabto_users(users_json):
    """
    Call this function from a django shell.
    Takes the path of json users dump
    """
    with open(users_json, 'r') as f:
        users = json.load(f)
        for fabuser in users:
            try:
               bio = fabuser['biografia']
            except KeyError:
               bio = fabuser['description']

            now = timezone.now()

            user, created = User.objects.get_or_create(username=fabuser['id'])
            if not created:
                continue

            up = UserProfile(
                user=user,
                name=fabuser['title'],
                address=fabuser['address'],
                tax_code=fabuser['fiscalCode'],
                vat_id=fabuser['vat'],
                bio=bio,
                endSubscription=now,
            )

            try:
                nfc_id = int(fabuser['rfid'], 16)
                card, _ = Card.objects.get_or_create(nfc_id=nfc_id)
                if card:
                    up.card = card
            except:
                print('Failed to convert rfid:', up.name, fabuser['rfid'])

            if 'immagine' in fabuser:
                with open(fabuser['immagine'], 'rb') as img:
                    f = File(img)
                    name = os.path.basename(fabuser['immagine'])
                    up.picture.save(name, f)

            try:
                up.save()
            except Exception as e:
                print(up.name, e)



from labadmin.models import (
    Card, UserProfile
)

from rest_framework import serializers


class UserProfileSerializer(serializers.ModelSerializer):

    class Meta:
        model = UserProfile
        fields = ('id','name')


class CardSerializer(serializers.ModelSerializer):
    class Meta:
        model = Card
        fields = ('nfc_id', 'credits')


class CardUpdateSerializer(serializers.Serializer):
    nfc_id = serializers.IntegerField()
    amount = serializers.IntegerField()

# class LogdoorSerializer(serializers.ModelSerializer):
#
#     class Meta:
#         model = Logdoor
#         fields = ('id', 'hour', 'user','doorOpened')
#
# class NfcSerializer(serializers.ModelSerializer):
#
#     def get_object(self):
#         try:
#             u = UserProfile.objects.get(nfcId=self.nfcId)
#             return u
#         except UserProfile.DoesNotExist:
#             return None
#
#     class Meta:
#         model = UserProfile
#         fields = ('id','nfcId')
#
# class UserProfileSerializer(serializers.ModelSerializer):
#
#     class Meta:
#         model = UserProfile
#         fields = ('id', 'username','utype', 'signup', 'subscriptionEnd', 'needSubcription', 'nfcId')

# class PermissionSerializer(serializers.ModelSerializer):
#
#     class Meta:
#         model = Permission
#         fields = ("id","user","devicetype","level")
#
#
# class LogdeviceSerializer(serializers.ModelSerializer):
#
#     class Meta:
#         model = Logdevice
#         fields = (
#             'id'
#             ,'user'
#             ,'device'
#             ,'bootDevice'
#             ,'shutdownDevice'
#             ,'startWork'
#             ,'finishWork'
#             ,'hourlyCost'
#         )



# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-09-21 16:35
from __future__ import unicode_literals

import datetime
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Category',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50)),
            ],
            options={
                'verbose_name_plural': 'categories',
            },
        ),
        migrations.CreateModel(
            name='Device',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100)),
                ('hourlyCost', models.FloatField(default=0.0)),
                ('mac', models.CharField(max_length=30)),
                ('category', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='labadmin.Category')),
            ],
        ),
        migrations.CreateModel(
            name='Group',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50)),
            ],
        ),
        migrations.CreateModel(
            name='LogAccess',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('datetime', models.DateTimeField(default=django.utils.timezone.now)),
                ('opened', models.BooleanField(default=False)),
            ],
        ),
        migrations.CreateModel(
            name='LogDevice',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('hourlyCost', models.FloatField(default=0.0)),
                ('bootDevice', models.DateTimeField()),
                ('startWork', models.DateTimeField()),
                ('finishWork', models.DateTimeField()),
                ('shutdownDevice', models.DateTimeField()),
                ('inWorking', models.BooleanField(default=True)),
                ('device', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='labadmin.Device')),
            ],
        ),
        migrations.CreateModel(
            name='LogError',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('datetime', models.DateTimeField(default=django.utils.timezone.now)),
                ('description', models.CharField(max_length=200)),
                ('code', models.CharField(blank=True, default='', max_length=200)),
            ],
        ),
        migrations.CreateModel(
            name='Payment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date', models.DateField(default=datetime.datetime.today)),
                ('value', models.FloatField(default=0.0)),
            ],
        ),
        migrations.CreateModel(
            name='Role',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50)),
                ('role_kind', models.IntegerField(choices=[(0, 'Door Access'), (1, 'Use Device')])),
                ('valid', models.BooleanField(default=True)),
                ('categories', models.ManyToManyField(blank=True, to='labadmin.Category')),
            ],
        ),
        migrations.CreateModel(
            name='TimeSlot',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50)),
                ('weekday_start', models.SmallIntegerField(choices=[(1, 'Monday'), (2, 'Tuesday'), (3, 'Wednesday'), (4, 'Thursday'), (5, 'Friday'), (6, 'Saturday'), (7, 'Sunday')], default=0)),
                ('weekday_end', models.SmallIntegerField(choices=[(1, 'Monday'), (2, 'Tuesday'), (3, 'Wednesday'), (4, 'Thursday'), (5, 'Friday'), (6, 'Saturday'), (7, 'Sunday')], default=0)),
                ('hour_start', models.TimeField()),
                ('hour_end', models.TimeField()),
            ],
        ),
        migrations.CreateModel(
            name='UserProfile',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=200)),
                ('address', models.CharField(blank=True, max_length=200, null=True)),
                ('tax_code', models.CharField(blank=True, max_length=50, null=True)),
                ('vat_id', models.CharField(blank=True, max_length=50, null=True)),
                ('picture', models.ImageField(blank=True, null=True, upload_to='labadmin/users/pictures')),
                ('bio', models.TextField(blank=True, null=True)),
                ('firstSignup', models.DateField(null=True)),
                ('lastSignup', models.DateField(null=True)),
                ('endSubscription', models.DateField()),
                ('needSubscription', models.BooleanField(default=True)),
                ('nfcId', models.BigIntegerField(null=True, unique=True)),
                ('groups', models.ManyToManyField(to='labadmin.Group')),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.AddField(
            model_name='role',
            name='time_slots',
            field=models.ManyToManyField(to='labadmin.TimeSlot'),
        ),
        migrations.AddField(
            model_name='payment',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='labadmin.UserProfile'),
        ),
        migrations.AddField(
            model_name='logdevice',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='labadmin.UserProfile'),
        ),
        migrations.AddField(
            model_name='logaccess',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='labadmin.UserProfile'),
        ),
        migrations.AddField(
            model_name='group',
            name='roles',
            field=models.ManyToManyField(to='labadmin.Role'),
        ),
    ]



from .settings import MQTT_CONFIG
from paho.mqtt import publish


def mqtt_publish(topic, payload):
    config = {k.lower(): v for k, v in MQTT_CONFIG.items()}
    publish.single(topic, payload, **config)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-10-12 12:23
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('labadmin', '0004_logcredits'),
    ]

    operations = [
        migrations.AlterField(
            model_name='userprofile',
            name='card',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='labadmin.Card'),
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import sys

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

version = '0.5.1'

setup(
    name='labadmin',
    version=version,
    description="""A django app to manage your Fablab""",
    author='Officine Torino',
    author_email='d.gomba@officine.cc',
    url='https://github.com/OfficineArduinoTorino/labadmin.git',
    packages=[
        'labadmin',
    ],
    include_package_data=True,
    install_requires=[
       'django<1.11,>=1.8',
       'djangorestframework>3,<3.7',
       'django-cors-middleware==1.3.1',
       'django-oauth-toolkit==0.10.0',
       'django-registration>=2.2,<2.3',
       'django-versatileimagefield>=1.7.0,<2',
       'paho-mqtt>1,<2',
       'Pillow<4.1',
    ],
    zip_safe=False,
    keywords='Fablab, lab, admin',
    license='MIT',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Framework :: Django',
        'Framework :: Django :: 1.8',
        'Framework :: Django :: 1.10',
        'Framework :: Django :: 1.11',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-05-18 21:58
from __future__ import unicode_literals

from django.db import migrations
import versatileimagefield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('labadmin', '0011_sketch'),
    ]

    operations = [
        migrations.AlterField(
            model_name='userprofile',
            name='picture',
            field=versatileimagefield.fields.VersatileImageField(blank=True, null=True, upload_to='labadmin/users/pictures'),
        ),
    ]



from django.conf import settings

from paho.mqtt.client import MQTTv311


BASE_MQTT_CONFIG = {
    'HOSTNAME': 'localhost',
    'PORT': 1883,
    'AUTH': None,
    'TLS': None,
    'PROTOCOL': MQTTv311,
    'TRANSPORT': 'tcp',
}
MQTT_CONFIG = {}


USER_MQTT_CONFIG = getattr(settings, 'LABADMIN_MQTT_CONFIG', {})

MQTT_CONFIG = BASE_MQTT_CONFIG.copy()
MQTT_CONFIG.update(USER_MQTT_CONFIG)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-10-11 16:29
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('labadmin', '0003_card_credits'),
    ]

    operations = [
        migrations.CreateModel(
            name='LogCredits',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('datetime', models.DateTimeField(default=django.utils.timezone.now)),
                ('amount', models.IntegerField()),
                ('from_admin', models.BooleanField(default=False)),
                ('card', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='labadmin.Card')),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



from django.db import models
from django.contrib.auth.models import User
from django.http import HttpResponse
from django.template import Template, Context
from django.utils import timezone
from django.utils.text import slugify

from versatileimagefield.fields import VersatileImageField

import uuid


class TimeSlotManager(models.Manager):
    def can_now(self):
        """Filters the queryset for this moment"""
        n = timezone.now()
        qs = self.get_queryset()
        return qs.filter(
            hour_start__lte=n.time(),
            hour_end__gte=n.time(),
            weekday_start__lte=n.isoweekday(),
            weekday_end__gte=n.isoweekday()
        )


class TimeSlot(models.Model):
    """
    TimeSlots are assigned to Roles
    """
    WEEKDAY_CHOICES = (
        (1,'Monday'),
        (2,'Tuesday'),
        (3,'Wednesday'),
        (4,'Thursday'),
        (5,'Friday'),
        (6,'Saturday'),
        (7,'Sunday')
    )
    name=models.CharField(max_length=50)
    weekday_start=models.SmallIntegerField(default=0, choices=WEEKDAY_CHOICES)
    weekday_end=models.SmallIntegerField(default=0, choices=WEEKDAY_CHOICES)
    hour_start=models.TimeField()
    hour_end=models.TimeField()

    objects = TimeSlotManager()

    def have_permission_now(self):
        n = timezone.now()
        return self.weekday_start <= n.weekday <= self.weekday_end and self.hour_start <= n.time <= self.hour_end

    def __str__(self):
        return "%s: %s - %s, %s - %s"%(self.name, self.WEEKDAY_CHOICES[self.weekday_start-1][1],self.WEEKDAY_CHOICES[self.weekday_end-1][1],self.hour_start,self.hour_end)


class Card(models.Model):
    """A card with a radio device"""
    nfc_id = models.BigIntegerField(unique=True)
    credits = models.IntegerField(default=0)

    def __str__(self):
        return "{}".format(self.nfc_id)

    def log_credits_update(self, amount, user=None, from_admin=False):
        LogCredits.objects.create(
            card=self,
            amount=amount,
            user=user,
            from_admin=from_admin,
        )


class UserProfile(models.Model):
    """
    The User Profile
    """
    user=models.OneToOneField(User)

    name=models.CharField(max_length=200)
    address = models.CharField(max_length=200, null=True, blank=True)
    tax_code = models.CharField(max_length=50, null=True, blank=True)
    vat_id = models.CharField(max_length=50, null=True, blank=True)
    picture = VersatileImageField(upload_to="labadmin/users/pictures", null=True, blank=True)
    bio = models.TextField(null=True, blank=True)

    firstSignup = models.DateField(null=True)
    lastSignup = models.DateField(null=True)
    endSubscription = models.DateField()
    needSubscription = models.BooleanField(default=True)

    card = models.OneToOneField(Card, null=True, blank=True)

    # define Many-To-Many fields
    groups=models.ManyToManyField('Group')

    def subscription_end(self):
        return self.endSubscription if self.needSubscription else "Subscription not needed"

    def has_valid_subscription(self):
        if not self.needSubscription:
            return True
        return self.needSubscription and self.endSubscription > timezone.now().date()

    def can_use_device_now(self, device):
        if not self.has_valid_subscription():
            return False
        roles = self.groups.values_list('roles__pk', flat=True).distinct()
        return TimeSlot.objects.can_now().filter(
            role__in=roles, role__valid=True, role__categories=device.category
        ).exists()

    def displaygroups(self):
        data = []
        for g in self.groups.all():
            data.append(g.name)
        return ",".join(data)

    def __str__(self):
        return self.name

class Category(models.Model):
    name=models.CharField(max_length=50)

    class Meta:
        verbose_name_plural = "categories"

    def __str__(self):
        return "%s"%(self.name)

class Group(models.Model):
    name=models.CharField(max_length=50)
    # define Many-To-Many fields
    roles=models.ManyToManyField('Role')

    def __str__(self):
        return self.name

class Role(models.Model):
    name=models.CharField(max_length=50)

    time_slots=models.ManyToManyField(TimeSlot)
    valid=models.BooleanField(default=True)

    # define Many-To-Many Fieds
    categories=models.ManyToManyField('Category',blank=True)

    def __str__(self):
        return self.name


class Sketch(models.Model):
    name = models.CharField(max_length=100)
    code = models.TextField()
    file_format = models.CharField(max_length=10, default='.txt')

    def __str__(self):
        return self.name

    def render(self, device, request):
        t = Template(self.code)
        c = Context({'device': device, 'user': request.user.userprofile})
        data = t.render(c)
        slug = slugify('{} {}'.format(device.name, self.name))
        filename = '{}{}'.format(slug, self.file_format)

        response = HttpResponse(data, content_type='text/plain')
        response['Content-Disposition'] = 'attachment; filename="{}"'.format(filename)
        return response

    class Meta:
        verbose_name_plural = 'Sketches'


class Device(models.Model):
    name = models.CharField(max_length=100)
    hourlyCost = models.FloatField(default=0.0)
    category = models.ForeignKey('Category')
    mac = models.CharField(max_length=30)
    token = models.CharField(max_length=64, null=True, blank=True)

    def __str__(self):
        return "%010d - %s" %(self.id, self.name)

    def last_activity(self):
        logaccess = self.logaccess_set.order_by('-id').first()
        if logaccess:
            return str(logaccess)
        logdevice = self.logdevice_set.order_by('-id').first()
        if not logdevice:
            return ''
        if logdevice.inWorking:
            return timezone.now()
        return logdevice.finishWork

    def save(self, *args, **kwargs):
        if not self.token:
            self.token = str(uuid.uuid4())
        super(Device, self).save(*args, **kwargs)


class DeviceUserCode(models.Model):
    """A code that permits access to a device for a specific user"""
    userprofile = models.ForeignKey(UserProfile)
    device = models.ForeignKey(Device)
    code = models.CharField(max_length=64)

    def __str__(self):
        return self.code

    class Meta:
        unique_together = (('userprofile', 'device', 'code'),)


class Payment(models.Model):
    date = models.DateField(default=timezone.now().today)
    value=models.FloatField(default=0.0)
    user=models.ForeignKey(UserProfile)

class LogError(models.Model):
    datetime = models.DateTimeField(default=timezone.now)
    description = models.CharField(max_length=200)
    code = models.CharField(default='', blank=True, max_length=200)
    device = models.ForeignKey(Device, null=True, blank=True)

class LogAccessManager(models.Manager):
    def log(self, card, opened, device):
        l = LogAccess.objects.create(card=card, opened=opened, device=device)
        return l

class LogAccess(models.Model):
    datetime = models.DateTimeField(default=timezone.now)
    card = models.ForeignKey(Card)
    opened = models.BooleanField(default=False)
    device = models.ForeignKey(Device, null=True, blank=True)

    objects= LogAccessManager()

    def __str__(self):
        return "{:%d/%m/%Y %H:%M} {} enter {}permitted".format(
            self.datetime, self.card.userprofile, "" if self.opened else "not "
        )

class LogDevice(models.Model):
    hourlyCost=models.FloatField(default=0.0)
    user=models.ForeignKey('UserProfile')
    device=models.ForeignKey('Device')
    bootDevice=models.DateTimeField()
    startWork=models.DateTimeField()
    finishWork=models.DateTimeField()
    shutdownDevice=models.DateTimeField()
    inWorking=models.BooleanField(default=True)

    def priceWork(self):
        if self.inWorking:
            return 'inWorking...'
        delta = self.finishWork - self.startWork
        duration = delta.total_seconds() / 3600
        return round(self.hourlyCost * duration, 2)

    def stop(self):
        n = timezone.now()
        self.inWorking = False
        self.finishWork = n
        self.shutdownDevice = n
        self.save(update_fields=['finishWork', 'shutdownDevice', 'inWorking'])

    def __str__(self):
        return "user: %s\ndevice: %s\nboot: %s\nstart: %s\ninWorking: %s\nHourlyCost: %f" %(self.user, self.device, self.bootDevice, self.startWork, "yes" if self.inWorking else "no\nshutdown: %s\nfinish: %s"%(self.shutdownDevice, self.finishWork), self.hourlyCost)


class LogCredits(models.Model):
    datetime = models.DateTimeField(default=timezone.now)
    card = models.ForeignKey(Card)
    amount = models.IntegerField()
    user = models.ForeignKey(User, null=True, blank=True)
    from_admin = models.BooleanField(default=False)

    def __str__(self):
        if self.from_admin:
            return '{} {} set credits for card {} to {} from admin'.format(
                self.datetime,
                self.user,
                self.card,
                self.amount
            )
        return '{} {} updated credits for card {} by {}'.format(
            self.datetime,
            self.user if self.user else '<AnonymousUser>',
            self.card,
            self.amount
        )
